@@ -1,15 +1,34 @@
 // Components for manipulating sequences of characters -*- C++ -*-
 
-// Copyright (C) 1997,1998 Cygnus Solutions
+// Copyright (C) 1997-1999 Cygnus Solutions
 //
-// This file is part of the libstdc++ version 3 distribution.
-//
-// This software is a copyrighted work licensed under the terms of the
-// Cygnus libstdc++ license. Please consult the file LICENSE.STD for
-// details.
+// This file is part of the GNU ISO C++ Library.  This library is free
+// software; you can redistribute it and/or modify it under the
+// terms of the GNU General Public License as published by the
+// Free Software Foundation; either version 2, or (at your option)
+// any later version.
+
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+
+// You should have received a copy of the GNU General Public License along
+// with this library; see the file COPYING.  If not, write to the Free
+// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// USA.
+
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.
 
 //
-// ISO C++ : 21 Strings library
+// ISO C++ 14882: 21 Strings library
 //
 
 #ifndef _CPP_BITS_STRING_H
@@ -32,10 +51,13 @@ namespace std {
     typedef typename _Alloc::const_reference const_reference;
     typedef typename _Alloc::pointer pointer;
     typedef typename _Alloc::const_pointer const_pointer;
-    typedef pointer iterator;
-    typedef const_pointer const_iterator;
+    typedef basic_string<_CharT, _Traits, _Alloc> string_type;
+    typedef __normal_iterator<pointer, string_type> iterator;
+    typedef __normal_iterator<const_pointer, string_type> const_iterator;
     typedef reverse_iterator<const_iterator> const_reverse_iterator;
     typedef reverse_iterator<iterator> reverse_iterator;
+    
+    // Data Members:
     static const size_type npos = ~(size_type)0;
 
   private:
@@ -70,8 +92,7 @@ namespace std {
 	    _M_refcopy () : _M_clone (__alloc1);  }
 
       // Create & Destroy
-      static const size_type _S_max_size =
-        ((npos - sizeof(_Rep) + 1) / sizeof(_CharT)) - 2;
+      static const size_type _S_max_size = (npos / sizeof(_CharT)) - 2;
       typedef typename _Alloc::rebind<char>::other _Raw_bytes_alloc;
       static _Rep* _S_create (size_t, const _Alloc&);
       void _M_dispose (const _Alloc& __a)
@@ -122,8 +143,8 @@ namespace std {
 
     // For the internal use we have functions similar to `begin'/`end'
     // but they do not call _M_leak.
-    iterator _M_ibegin () const { return _M_data (); }
-    iterator _M_iend () const { return _M_data () + this->size (); }
+    iterator _M_ibegin () const { return iterator (_M_data ()); }
+    iterator _M_iend () const { return iterator (_M_data () + this->size ()); }
 
     void _M_leak ()    // for use in begin() & non-const op[]
       { if (_M_rep ()->_M_state >= 0) _M_leak_hard(); }
@@ -141,7 +162,10 @@ namespace std {
     template <class _Iterator>
       static void
       _S_copy_chars(_CharT* __p, _Iterator __j1, _Iterator __j2)
-        { for (; __j1 != __j2; ++__j1, ++__p) _Traits::assign (*__p, *__j1); }
+        { 
+	  for (; __j1 != __j2; ++__j1, ++__p) 
+	    _Traits::assign (*__p, *__j1); //these types are off
+	}
 
     static void
     _S_copy_chars(_CharT* __p, const _CharT* __j1, const _CharT* __j2)
@@ -191,10 +215,15 @@ namespace std {
       { return assign (1, __c); }
 
     // Iterators:
-    iterator begin () { _M_leak (); return &this->operator[](0); }
-    const_iterator begin () const { return &(*_M_rep ())[0]; }
-    iterator end () { _M_leak (); return &(this->operator[](length ())); }
-    const_iterator end () const { return &(*_M_rep ())[length ()]; }
+    iterator begin () { _M_leak (); return iterator (&this->operator[](0)); }
+    const_iterator begin () const { return const_iterator (&(*_M_rep ())[0]); }
+    iterator end ()
+      {
+         _M_leak ();
+         return iterator (&(this->operator[](length ())));
+      }
+    const_iterator end () const
+      { return const_iterator (&(*_M_rep ())[length ()]); }
 
     reverse_iterator rbegin ()
       { return reverse_iterator (end ()); }
@@ -246,7 +275,7 @@ namespace std {
     basic_string& operator += (const _CharT* __s)
       { return append (__s); }
     basic_string& operator += (_CharT __c)
-      { return append (1, __c); }
+      { return append (size_type(1), __c); }
 
 
     basic_string& append (const basic_string& __str)
@@ -292,17 +321,22 @@ namespace std {
 		      __str._M_fold (__pos2, __n));
         return *this; }
     basic_string& insert (size_type __pos, const _CharT* __s, size_type __n)
-      { this->insert (_M_check (__pos), __s, __s+__n);
-        return *this; }
+      { 
+	this->insert (_M_check (__pos), __s, __s + __n);
+        return *this; 
+      }
     basic_string& insert (size_type __pos, const _CharT* __s)
       { return this->insert (__pos, __s, _Traits::length (__s)); }
     basic_string& insert (size_type __pos, size_type __n, _CharT __c)
-      { this->insert (_M_check (__pos), __n, __c); return *this; }
+      { 
+	this->insert (_M_check (__pos), __n, __c); 
+	return *this; 
+      }
     iterator insert (iterator __p, _CharT __c = _CharT ())
       {
-	size_type __off = __p - _M_ibegin ();
-	this->insert (__off, __c, 1);
-	return this->_M_ibegin () + __off;
+	size_type __pos = __p - _M_ibegin ();
+	this->insert (__pos, size_type(1), __c);
+	return this->_M_ibegin () + __pos; 
       }
     void insert (iterator __p, size_type __n, _CharT __c)
       {	this->replace (__p, __p, __n, __c);  }
@@ -534,7 +568,7 @@ namespace std {
   operator+ (const basic_string<_CharT,_Traits,_Alloc>& __lhs, _CharT __rhs)
   {
     basic_string <_CharT,_Traits,_Alloc> __str (__lhs);
-    __str.append (1, __rhs);
+    __str.append (basic_string<_CharT,_Traits,_Alloc>::size_type(1), __rhs);
     return __str;
   }
 
@@ -721,7 +755,6 @@ namespace std {
     return getline (__is, __str, __is.widen ('\n'));
   }
 
-
 } // namespace std
 
 #endif /* _CPP_BITS_STRING_H */
