@@ -54,8 +54,8 @@ cexp (__complex__ double x)
 	    }
 	  else
 	    {
-	      __real__ retval = __copysign (exp_val, cosix);
-	      __imag__ retval = __copysign (exp_val, sinix);
+	      __real__ retval = copysign (exp_val, cosix);
+	      __imag__ retval = copysign (exp_val, sinix);
 	    }
 	}
       else
