@@ -1,12 +1,29 @@
-// locale.cc
+// Copyright (C) 1997-1999 Cygnus Solutions
+//
+// This file is part of the GNU ISO C++ Library.  This library is free
+// software; you can redistribute it and/or modify it under the
+// terms of the GNU General Public License as published by the
+// Free Software Foundation; either version 2, or (at your option)
+// any later version.
 
-// Copyright (C) 1998 Cygnus Solutions
-//
-// This file is part of the libstdc++ version 3 distribution.
-//
-// This software is a copyrighted work licensed under the terms of the
-// Cygnus libstdc++ license. Please consult the file LICENSE.STD for
-// details.
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+
+// You should have received a copy of the GNU General Public License along
+// with this library; see the file COPYING.  If not, write to the Free
+// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// USA.
+
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.
 
 #include <bits/std_clocale.h>
 #include <bits/std_cstring.h>
@@ -15,10 +32,9 @@
 #include <bits/std_exception.h>
 #include <bits/std_stdexcept.h>
 #include <bits/std_locale.h>
+#include <bits/std_vector.h>
 
-#if _G_USE_NAMESPACE
 namespace std {
-#endif
 
 // Implementation of locale::_Impl members
 //////////////////////////////////////////
@@ -27,41 +43,80 @@ namespace std {
   // destructor
   locale::_Impl::~_Impl () throw ()
   {
-    std::vector<facet*>::iterator it = _M_facets.begin ();
-    for (; it != _M_facets.end (); ++it)
-      (*it)->_M_remove_reference ();
+    std::vector<facet*>::iterator it = _M_facets->begin();
+    for (; it != _M_facets->end(); ++it)
+      (*it)->_M_remove_reference();
+    delete _M_facets;
+    delete _M_category_names;
   }
 
   // copy constructors
   locale::_Impl::_Impl (size_t numfacets, size_t refs)
   : _M_num_references (refs-1)
-  , _M_facets (numfacets, (facet*)0)
-  , _M_category_names (_S_num_categories, string ("*"))
+    //  , _M_facets (numfacets, (facet*)0)
+    //  , _M_category_names (_S_num_categories, string ("*"))
   , _M_has_name (false)
   , _M_cached_name_ok (false)
   , _M_cached_name (string ("*"))
-  { }
+  { 
+#if 1
+    typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
+    typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
+    try {
+      _M_facets = new __vec_facet (numfacets, (facet*)0);
+    }
+    catch (...) {
+      delete _M_facets;
+      throw;
+    }
+    try {
+       _M_category_names = new __vec_string (_S_num_categories, string ("*"));
+    }
+    catch (...) {
+      delete _M_category_names;
+      throw;
+    }
+#endif
+  }
   
   locale::_Impl::_Impl (const _Impl& other,
 			size_t refs)
   : _M_num_references (refs)
-  , _M_facets (other._M_facets)
-  , _M_category_names (other._M_category_names)
+    //  , _M_facets (other._M_facets)
+    //  , _M_category_names (other._M_category_names)
   , _M_has_name (other._M_has_name)
   , _M_cached_name_ok (other._M_cached_name_ok)
   , _M_cached_name (other._M_cached_name)
   {
-    std::vector<facet*>::iterator it = _M_facets.begin ();
-    for (; it != _M_facets.end (); ++it)
-      (*it)->_M_add_reference ();
+#if 1
+    typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
+    typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
+    try {
+      _M_facets = new __vec_facet  (*(other._M_facets));
+    }
+    catch (...) {
+      delete _M_facets;
+      throw;
+    }
+    try {
+       _M_category_names = new __vec_string (*(other._M_category_names));
+    }
+    catch (...) {
+      delete _M_category_names;
+      throw;
+    }
+#endif
+
+    std::vector<facet*>::iterator it = _M_facets->begin();
+    for (; it != _M_facets->end(); ++it)
+      (*it)->_M_add_reference();
   }
   
   //////////
   void
-  locale::_Impl::_M_replace_categories (const _Impl* other,
-					category cats)
+  locale::_Impl::_M_replace_categories(const _Impl* other, category cats)
   {
-    assert((cats & locale::all) && !(cats && ~locale::all));
+    assert((cats & locale::all) && !(cats & ~locale::all));
     
     unsigned mask = (locale::all & -(unsigned)locale::all);
     for (unsigned ix = 0; (-mask & cats) != 0; ++ix, (mask <<= 1))
@@ -69,7 +124,7 @@ namespace std {
 	if (mask & cats)
 	  {
 	    _M_replace_category (other, _S_facet_categories[ix]);
-	    _M_category_names[ix] = other->_M_category_names[ix];
+	    (*_M_category_names)[ix] = (*(other->_M_category_names))[ix];
 	  }
       }
   }
@@ -90,11 +145,11 @@ namespace std {
   {
     size_t index = idp->_M_index;
     if (index == 0 ||
-	other->_M_facets.size () <= index ||
-	other->_M_facets[index] == 0)
+	other->_M_facets->size() <= index ||
+	(*(other->_M_facets))[index] == 0)
       throw runtime_error ("no locale facet");
 	
-    _M_install_facet (idp, other->_M_facets[index]); 
+    _M_install_facet (idp, (*(other->_M_facets))[index]); 
   }
 
   //////////
@@ -109,12 +164,12 @@ namespace std {
     if (!index)
       index = ++locale::id::_S_highwater;  // XXX MT
 
-    if (index >= _M_facets.size ())
-      _M_facets.resize (index+1, 0);  // might throw
-    facet*& fpr = _M_facets[index];
+    if (index >= _M_facets->size())
+      _M_facets->resize(index+1, 0);  // might throw
+    facet*& fpr = (*_M_facets)[index];
     // order matters, here:
-    fp->_M_add_reference ();
-    if (fpr) fpr->_M_remove_reference ();
+    fp->_M_add_reference();
+    if (fpr) fpr->_M_remove_reference();
     fpr = fp;
   }
 
@@ -403,9 +458,9 @@ namespace std {
   const int* const& ctype<char>::_S_tolower = __ctype_tolower;
   const ctype_base::mask* const& ctype<char>::_S_table = __ctype_b;
 #else
-  const int* const& ctype<char>::_S_toupper = (const int* const&)0;
-  const int* const& ctype<char>::_S_tolower = (const int* const&)0;
-  const ctype_base::mask* const& ctype<char>::_S_table = (const ctype_base::mask* const&)0;
+  const int* const& ctype<char>::_S_toupper = 0;
+  const int* const& ctype<char>::_S_tolower = 0;
+  const ctype_base::mask* const& ctype<char>::_S_table = 0;
 #endif
   
   char
@@ -479,15 +534,16 @@ namespace std {
   
   ////////////
   locale::id ctype<wchar_t>::id;
-#ifdef __linux__ 
-  // XXX these are Glibc-specific.
-  const int*& ctype<wchar_t>::_S_toupper = __ctype_toupper;
-  const int*& ctype<wchar_t>::_S_tolower = __ctype_tolower;
-  const ctype_base::mask*& ctype<wchar_t>::_S_table = __ctype_b;
+#if defined(__GLIBC__) && (__GLIBC__ >= 2)
+
+  // XXX These are Glibc-specific.
+  const int* const& ctype<wchar_t>::_S_toupper = __ctype_toupper;
+  const int* const& ctype<wchar_t>::_S_tolower = __ctype_tolower;
+  const ctype_base::mask* const& ctype<wchar_t>::_S_table = __ctype_b;
 #else
-  const int*& ctype<wchar_t>::_S_toupper = (const int*&)0;
-  const int*& ctype<wchar_t>::_S_tolower = (const int*&)0;
-  const ctype_base::mask*& ctype<wchar_t>::_S_table = (const ctype_base::mask*&)0;
+  const int* const& ctype<wchar_t>::_S_toupper = 0;
+  const int* const& ctype<wchar_t>::_S_tolower = 0;
+  const ctype_base::mask* const& ctype<wchar_t>::_S_table = 0;
 #endif
 
   ctype<wchar_t>::ctype (size_t refs)
@@ -618,7 +674,7 @@ namespace std {
   : ctype<wchar_t> (refs)
   {
   }
-  
+ 
 
   ///////////
   locale::id codecvt<char,char,mbstate_t>::id;
@@ -631,12 +687,20 @@ namespace std {
   codecvt<char,char,mbstate_t>::~codecvt () { }
   
   codecvt_base::result
-  codecvt<char,char,mbstate_t>::do_out (state_type& state,
-	      const intern_type* from, const intern_type* from_end,
-	      const intern_type*& from_next,
-	      extern_type* to, extern_type* to_limit,
-	      extern_type*& to_next) const
-    { from_next = from; to_next = to; return noconv;  }
+  codecvt<char,char,mbstate_t>::do_out (state_type& __state,
+	      const intern_type* __from, const intern_type* __from_end,
+	      const intern_type*& __from_next,
+	      extern_type* __to, extern_type* __to_end,
+	      extern_type*& __to_next) const
+    { 
+      size_t __sizefrom = __from_end - __from;
+      size_t __sizeto = __to_end - __to;
+      size_t __length = __sizefrom <= __sizeto ? __sizefrom : __sizeto;
+      memcpy(__to, __from, __length);
+      __from_next = __from; 
+      __to_next = __to;
+      return noconv;  
+    }
   
   codecvt_base::result
   codecvt<char,char,mbstate_t>::do_unshift (state_type& state,
@@ -645,12 +709,20 @@ namespace std {
     { to_next = to; return noconv; }
   
   codecvt_base::result
-  codecvt<char,char,mbstate_t>::do_in (state_type& state,
-	     const extern_type* from, const extern_type* from_end,
-	     const extern_type*& from_next,
-	     intern_type* to, intern_type* to_limit,
-	     intern_type*& to_next) const
-    { from_next = from; to_next = to; return noconv;  }
+  codecvt<char,char,mbstate_t>::do_in (state_type& __state,
+	     const extern_type* __from, const extern_type* __from_end,
+	     const extern_type*& __from_next,
+	     intern_type* __to, intern_type* __to_end,
+	     intern_type*& __to_next) const
+    { 
+      size_t __sizefrom = __from_end - __from;
+      size_t __sizeto = __to_end - __to;
+      size_t __length = __sizefrom <= __sizeto ? __sizefrom : __sizeto;
+      memcpy(__to, __from, __length);
+      __from_next = __from; 
+      __to_next = __to;
+      return noconv;  
+    }
   
 
   int 
@@ -933,6 +1005,4 @@ namespace std {
   {
   }
 
-#if _G_USE_NAMESPACE
 } // namespace std
-#endif
