@@ -29,33 +29,28 @@
  */
 
 
-#ifndef _CPP_BITS_STL_ALGOBASE_H
-#define _CPP_BITS_STL_ALGOBASE_H 1
+#ifndef __SGI_STL_INTERNAL_ALGOBASE_H
+#define __SGI_STL_INTERNAL_ALGOBASE_H
 
-/* XXX Why all the redundant guards? */
-#ifndef _CPP_BITS_STL_CONFIG_H
 #include <bits/stl_config.h>
-#endif
-#ifndef _CPP_BITS_STL_RELOPS
 #include <bits/stl_relops.h>
-#endif
-#ifndef _CPP_BITS_STL_PAIR_H
 #include <bits/stl_pair.h>
-#endif
-#ifndef _CPP_BITS_TYPE_TRAITS_H_
 #include <bits/type_traits.h>
-#endif
 
 #include <bits/std_cstring.h>
 #include <bits/std_climits.h>
 #include <bits/std_cstdlib.h>
 #include <bits/std_cstddef.h>
 #include <bits/std_new.h>
-#include <bits/std_iosfwd.h>
 
-#ifndef _CPP_BITS_STL_ITERATOR_H
+#ifdef __STL_USE_NEW_IOSTREAMS 
+#include <iosfwd>
+#else /* __STL_USE_NEW_IOSTREAMS */
+#include <bits/std_iostream.h>
+#endif /* __STL_USE_NEW_IOSTREAMS */
+
+#include <bits/stl_iterator_base.h>
 #include <bits/stl_iterator.h>
-#endif
 
 __STL_BEGIN_NAMESPACE
 
@@ -387,7 +382,7 @@ template <class _InputIter1, class _InputIter2>
 inline bool equal(_InputIter1 __first1, _InputIter1 __last1,
                   _InputIter2 __first2) {
   for ( ; __first1 != __last1; ++__first1, ++__first2)
-    if (! (*__first1 == *__first2)) // BK hack
+    if (*__first1 != *__first2)
       return false;
   return true;
 }
@@ -520,7 +515,7 @@ int lexicographical_compare_3way(_InputIter1 __first1, _InputIter1 __last1,
 
 __STL_END_NAMESPACE
 
-#endif /* _CPP_BITS_STL_ALGOBASE_H */
+#endif /* __SGI_STL_INTERNAL_ALGOBASE_H */
 
 // Local Variables:
 // mode:C++
