@@ -28,8 +28,8 @@
  *   You should not attempt to use it directly.
  */
 
-#ifndef _CPP_BITS_STL_DEQUE_H
-#define _CPP_BITS_STL_DEQUE_H 1
+#ifndef __SGI_STL_INTERNAL_DEQUE_H
+#define __SGI_STL_INTERNAL_DEQUE_H
 
 /* Class invariants:
  *  For any nonsingular iterator i:
@@ -208,6 +208,9 @@ struct _Deque_iterator {
     return (_M_node == __x._M_node) ? 
       (_M_cur < __x._M_cur) : (_M_node < __x._M_node);
   }
+  bool operator>(const _Self& __x) const  { return __x < *this; }
+  bool operator<=(const _Self& __x) const { return !(__x < *this); }
+  bool operator>=(const _Self& __x) const { return !(*this < __x); }
 
   void _M_set_node(_Map_pointer __new_node) {
     _M_node = __new_node;
@@ -274,29 +277,30 @@ template <class _Tp, class _Alloc, size_t __bufsiz, bool __is_static>
 class _Deque_alloc_base {
 public:
   typedef typename _Alloc_traits<_Tp,_Alloc>::allocator_type allocator_type;
-  allocator_type get_allocator() const { return node_allocator; }
+  allocator_type get_allocator() const { return _M_node_allocator; }
 
   _Deque_alloc_base(const allocator_type& __a)
-    : node_allocator(__a), map_allocator(__a), _M_map(0), _M_map_size(0)
+    : _M_node_allocator(__a), _M_map_allocator(__a),
+      _M_map(0), _M_map_size(0)
   {}
   
 protected:
   typedef typename _Alloc_traits<_Tp*, _Alloc>::allocator_type
-          map_allocator_type;
+          _Map_allocator_type;
 
-  allocator_type node_allocator;
-  map_allocator_type map_allocator;
+  allocator_type      _M_node_allocator;
+  _Map_allocator_type _M_map_allocator;
 
   _Tp* _M_allocate_node() {
-    return node_allocator.allocate(__deque_buf_size(__bufsiz,sizeof(_Tp)));
+    return _M_node_allocator.allocate(__deque_buf_size(__bufsiz,sizeof(_Tp)));
   }
   void _M_deallocate_node(_Tp* __p) {
-    node_allocator.deallocate(__p, __deque_buf_size(__bufsiz,sizeof(_Tp)));
+    _M_node_allocator.deallocate(__p, __deque_buf_size(__bufsiz,sizeof(_Tp)));
   }
   _Tp** _M_allocate_map(size_t __n) 
-    { return map_allocator.allocate(__n); }
+    { return _M_map_allocator.allocate(__n); }
   void _M_deallocate_map(_Tp** __p, size_t __n) 
-    { map_allocator.deallocate(__p, __n); }
+    { _M_map_allocator.deallocate(__p, __n); }
 
   _Tp** _M_map;
   size_t _M_map_size;
@@ -316,12 +320,14 @@ protected:
   typedef typename _Alloc_traits<_Tp, _Alloc>::_Alloc_type _Node_alloc_type;
   typedef typename _Alloc_traits<_Tp*, _Alloc>::_Alloc_type _Map_alloc_type;
 
-  _Tp* _M_allocate_node()
-    { return _Node_alloc_type::allocate(__deque_buf_size(__bufsiz, 
-                                                         sizeof(_Tp))); }
-  void _M_deallocate_node(_Tp* __p)
-    { _Node_alloc_type::deallocate(__p, __deque_buf_size(__bufsiz, 
-                                                         sizeof(_Tp))); }
+  _Tp* _M_allocate_node() {
+    return _Node_alloc_type::allocate(__deque_buf_size(__bufsiz,
+                                                       sizeof(_Tp)));
+  }
+  void _M_deallocate_node(_Tp* __p) {
+    _Node_alloc_type::deallocate(__p, __deque_buf_size(__bufsiz, 
+                                                       sizeof(_Tp)));
+  }
   _Tp** _M_allocate_map(size_t __n) 
     { return _Map_alloc_type::allocate(__n); }
   void _M_deallocate_map(_Tp** __p, size_t __n) 
@@ -342,7 +348,7 @@ public:
           _Base;
   typedef typename _Base::allocator_type allocator_type;
   typedef _Deque_iterator<_Tp,_Tp&,_Tp*,__bufsiz>              iterator;
-  typedef _Deque_iterator<_Tp,const _Tp&,const _Tp&, __bufsiz>  const_iterator;
+  typedef _Deque_iterator<_Tp,const _Tp&,const _Tp*, __bufsiz> const_iterator;
 
   _Deque_base(const allocator_type& __a, size_t __num_elements)
     : _Base(__a), _M_start(), _M_finish()
@@ -371,8 +377,8 @@ public:
   typedef _Deque_iterator<_Tp,_Tp&,_Tp*,__bufsiz>              iterator;
   typedef _Deque_iterator<_Tp,const _Tp&,const _Tp*, __bufsiz> const_iterator;
 #else /* __STL_NON_TYPE_TMPL_PARAM_BUG */
-  typedef _Deque_iterator<_Tp,_Tp&,_Tp*>                      iterator;
-  typedef _Deque_iterator<_Tp,const _Tp&,const _Tp*>          const_iterator;
+  typedef _Deque_iterator<_Tp,_Tp&,_Tp*>                       iterator;
+  typedef _Deque_iterator<_Tp,const _Tp&,const _Tp*>           const_iterator;
 #endif /* __STL_NON_TYPE_TMPL_PARAM_BUG */
 
   typedef _Alloc allocator_type;
@@ -453,7 +459,7 @@ _Deque_base<_Tp,_Alloc,__bufsiz>::_M_initialize_map(size_t __num_elements)
 template <class _Tp, class _Alloc, size_t __bufsiz>
 void
 _Deque_base<_Tp,_Alloc,__bufsiz>::_M_create_nodes(_Tp** __nstart,
-                                                 _Tp** __nfinish)
+                                                  _Tp** __nfinish)
 {
   _Tp** __cur;
   __STL_TRY {
@@ -466,7 +472,7 @@ _Deque_base<_Tp,_Alloc,__bufsiz>::_M_create_nodes(_Tp** __nstart,
 template <class _Tp, class _Alloc, size_t __bufsiz>
 void 
 _Deque_base<_Tp,_Alloc,__bufsiz>::_M_destroy_nodes(_Tp** __nstart,
-                                                  _Tp** __nfinish)
+                                                   _Tp** __nfinish)
 {
   for (_Tp** __n = __nstart; __n < __nfinish; ++__n)
     _M_deallocate_node(*__n);
@@ -938,6 +944,15 @@ public:
   bool operator<(const deque<_Tp,_Alloc,0>& __x) const {
     return lexicographical_compare(begin(), end(), __x.begin(), __x.end());
   }
+  bool operator>(const deque<_Tp,_Alloc,0>& __x) const {
+    return __x < *this;
+  }
+  bool operator<=(const deque<_Tp,_Alloc,0>& __x) const {
+    return !(__x < *this);
+  }
+  bool operator>=(const deque<_Tp,_Alloc,0>& __x) const {
+    return !(*this < __x);
+  }
 #endif /* __STL_NON_TYPE_TMPL_PARAM_BUG */
 };
 
@@ -964,17 +979,24 @@ void deque<_Tp, _Alloc, __bufsize>
 template <class _Tp, class _Alloc, size_t __bufsize>
 void 
 deque<_Tp, _Alloc, __bufsize>::insert(iterator __pos,
-                                     size_type __n, const value_type& __x)
+                                      size_type __n, const value_type& __x)
 {
   if (__pos._M_cur == _M_start._M_cur) {
     iterator __new_start = _M_reserve_elements_at_front(__n);
-    uninitialized_fill(__new_start, _M_start, __x);
-    _M_start = __new_start;
+    __STL_TRY {
+      uninitialized_fill(__new_start, _M_start, __x);
+      _M_start = __new_start;
+    }
+    __STL_UNWIND(_M_destroy_nodes(__new_start._M_node, _M_start._M_node));
   }
   else if (__pos._M_cur == _M_finish._M_cur) {
     iterator __new_finish = _M_reserve_elements_at_back(__n);
-    uninitialized_fill(_M_finish, __new_finish, __x);
-    _M_finish = __new_finish;
+    __STL_TRY {
+      uninitialized_fill(_M_finish, __new_finish, __x);
+      _M_finish = __new_finish;
+    }
+    __STL_UNWIND(_M_destroy_nodes(_M_finish._M_node + 1, 
+                                  __new_finish._M_node + 1));    
   }
   else 
     _M_insert_aux(__pos, __n, __x);
@@ -984,8 +1006,8 @@ deque<_Tp, _Alloc, __bufsize>::insert(iterator __pos,
 
 template <class _Tp, class _Alloc, size_t __bufsize>
 void deque<_Tp, _Alloc, __bufsize>::insert(iterator __pos,
-                                      const value_type* __first,
-                                      const value_type* __last) {
+                                           const value_type* __first,
+                                           const value_type* __last) {
   size_type __n = __last - __first;
   if (__pos._M_cur == _M_start._M_cur) {
     iterator __new_start = _M_reserve_elements_at_front(__n);
@@ -1010,8 +1032,8 @@ void deque<_Tp, _Alloc, __bufsize>::insert(iterator __pos,
 
 template <class _Tp, class _Alloc, size_t __bufsize>
 void deque<_Tp,_Alloc,__bufsize>::insert(iterator __pos,
-                                        const_iterator __first,
-                                        const_iterator __last)
+                                         const_iterator __first,
+                                         const_iterator __last)
 {
   size_type __n = __last - __first;
   if (__pos._M_cur == _M_start._M_cur) {
@@ -1107,20 +1129,23 @@ template <class _Tp, class _Alloc, size_t __bufsize>
 template <class _InputIterator>
 void
 deque<_Tp,_Alloc,__bufsize>::_M_range_initialize(_InputIterator __first,
-                                                _InputIterator __last,
-                                                input_iterator_tag)
+                                                 _InputIterator __last,
+                                                 input_iterator_tag)
 {
   _M_initialize_map(0);
-  for ( ; __first != __last; ++__first)
-    push_back(*__first);
+  __STL_TRY {
+    for ( ; __first != __last; ++__first)
+      push_back(*__first);
+  }
+  __STL_UNWIND(clear());
 }
 
 template <class _Tp, class _Alloc, size_t __bufsize>
 template <class _ForwardIterator>
 void
 deque<_Tp,_Alloc,__bufsize>::_M_range_initialize(_ForwardIterator __first,
-                                                _ForwardIterator __last,
-                                                forward_iterator_tag)
+                                                 _ForwardIterator __last,
+                                                 forward_iterator_tag)
 {
   size_type __n = 0;
   distance(__first, __last, __n);
@@ -1236,9 +1261,9 @@ template <class _Tp, class _Alloc, size_t __bufsize>
 template <class _InputIterator>
 void 
 deque<_Tp,_Alloc,__bufsize>::insert(iterator __pos,
-                                   _InputIterator __first,
-                                   _InputIterator __last,
-                                   input_iterator_tag)
+                                    _InputIterator __first,
+                                    _InputIterator __last,
+                                    input_iterator_tag)
 {
   copy(__first, __last, inserter(*this, __pos));
 }
@@ -1247,9 +1272,9 @@ template <class _Tp, class _Alloc, size_t __bufsize>
 template <class _ForwardIterator>
 void 
 deque<_Tp,_Alloc,__bufsize>::insert(iterator __pos,
-                                   _ForwardIterator __first,
-                                   _ForwardIterator __last,
-                                   forward_iterator_tag) {
+                                    _ForwardIterator __first,
+                                    _ForwardIterator __last,
+                                    forward_iterator_tag) {
   size_type __n = 0;
   distance(__first, __last, __n);
   if (__pos._M_cur == _M_start._M_cur) {
@@ -1278,7 +1303,7 @@ deque<_Tp,_Alloc,__bufsize>::insert(iterator __pos,
 template <class _Tp, class _Alloc, size_t __bufsize>
 typename deque<_Tp, _Alloc, __bufsize>::iterator
 deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
-                                          const value_type& __x)
+                                           const value_type& __x)
 {
   difference_type __index = __pos - _M_start;
   value_type __x_copy = __x;
@@ -1338,8 +1363,8 @@ deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos)
 template <class _Tp, class _Alloc, size_t __bufsize>
 void
 deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
-                                          size_type __n,
-                                          const value_type& __x)
+                                           size_type __n,
+                                           const value_type& __x)
 {
   const difference_type __elems_before = __pos - _M_start;
   size_type __length = size();
@@ -1397,9 +1422,9 @@ template <class _Tp, class _Alloc, size_t __bufsize>
 template <class _ForwardIterator>
 void
 deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
-                                       _ForwardIterator __first,
-                                       _ForwardIterator __last,
-                                       size_type __n)
+                                           _ForwardIterator __first,
+                                           _ForwardIterator __last,
+                                           size_type __n)
 {
   const difference_type __elemsbefore = __pos - _M_start;
   size_type __length = size();
@@ -1458,9 +1483,9 @@ deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
 template <class _Tp, class _Alloc, size_t __bufsize>
 void 
 deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
-                                       const value_type* __first,
-                                       const value_type* __last,
-                                       size_type __n)
+                                           const value_type* __first,
+                                           const value_type* __last,
+                                           size_type __n)
 {
   const difference_type __elemsbefore = __pos - _M_start;
   size_type __length = size();
@@ -1516,9 +1541,9 @@ deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
 template <class _Tp, class _Alloc, size_t __bufsize>
 void
 deque<_Tp,_Alloc,__bufsize>::_M_insert_aux(iterator __pos,
-                                       const_iterator __first,
-                                       const_iterator __last,
-                                       size_type __n)
+                                           const_iterator __first,
+                                           const_iterator __last,
+                                           size_type __n)
 {
   const difference_type __elemsbefore = __pos - _M_start;
   size_type __length = size();
@@ -1656,59 +1681,49 @@ deque<_Tp,_Alloc,__bufsize>::_M_reallocate_map(size_type __nodes_to_add,
 
 template <class _Tp, class _Alloc, size_t __bufsiz>
 inline bool operator==(const deque<_Tp, _Alloc, __bufsiz>& __x,
-		       const deque<_Tp, _Alloc, __bufsiz>& __y)
+                       const deque<_Tp, _Alloc, __bufsiz>& __y)
 {
   return __x.size() == __y.size() &&
          equal(__x.begin(), __x.end(), __y.begin());
 }
 
-// CYGNUS LOCAL 8/18/1998
-template <class _Tp, class _Alloc, size_t __bufsiz>
-inline bool operator!=(const deque<_Tp, _Alloc, __bufsiz>& __x,
-		       const deque<_Tp, _Alloc, __bufsiz>& __y)
-{
-  return ! (__x == __y);
-}
-
 template <class _Tp, class _Alloc, size_t __bufsiz>
 inline bool operator<(const deque<_Tp, _Alloc, __bufsiz>& __x,
-		      const deque<_Tp, _Alloc, __bufsiz>& __y)
+                      const deque<_Tp, _Alloc, __bufsiz>& __y)
 {
   return lexicographical_compare(__x.begin(), __x.end(), 
                                  __y.begin(), __y.end());
 }
 
-// CYGNUS LOCAL 8/18/1998
+#ifdef __STL_FUNCTION_TMPL_PARTIAL_ORDER
+
+template <class _Tp, class _Alloc, size_t __bufsiz>
+inline bool operator!=(const deque<_Tp, _Alloc, __bufsiz>& __x,
+                       const deque<_Tp, _Alloc, __bufsiz>& __y)
+{
+  return !(__x == __y);
+}
+
 template <class _Tp, class _Alloc, size_t __bufsiz>
 inline bool operator>(const deque<_Tp, _Alloc, __bufsiz>& __x,
-		      const deque<_Tp, _Alloc, __bufsiz>& __y)
+                      const deque<_Tp, _Alloc, __bufsiz>& __y)
 {
   return __y < __x;
 }
 
-// CYGNUS LOCAL 8/18/1998
 template <class _Tp, class _Alloc, size_t __bufsiz>
-inline bool 
-operator<=(const deque<_Tp, _Alloc, __bufsiz>& __x,
-	   const deque<_Tp, _Alloc, __bufsiz>& __y)
-{ 
-  return !(__x > __y);
+inline bool operator<=(const deque<_Tp, _Alloc, __bufsiz>& __x,
+                       const deque<_Tp, _Alloc, __bufsiz>& __y)
+{
+  return !(__y < __x);
 }
-
-// CYGNUS LOCAL 8/18/1998
 template <class _Tp, class _Alloc, size_t __bufsiz>
-inline bool 
-operator>=(const deque<_Tp, _Alloc, __bufsiz>& __x,
-	   const deque<_Tp, _Alloc, __bufsiz>& __y)
+inline bool operator>=(const deque<_Tp, _Alloc, __bufsiz>& __x,
+                       const deque<_Tp, _Alloc, __bufsiz>& __y)
 {
   return !(__x < __y);
 }
 
-#endif /* __STL_NON_TYPE_TMPL_PARAM_BUG */
-
-#if defined(__STL_FUNCTION_TMPL_PARTIAL_ORDER) && \
-    !defined(__STL_NON_TYPE_TMPL_PARAM_BUG)
-
 template <class _Tp, class _Alloc, size_t __bufsiz>
 inline void 
 swap(deque<_Tp,_Alloc,__bufsiz>& __x, deque<_Tp,_Alloc,__bufsiz>& __y)
@@ -1716,7 +1731,8 @@ swap(deque<_Tp,_Alloc,__bufsiz>& __x, deque<_Tp,_Alloc,__bufsiz>& __y)
   __x.swap(__y);
 }
 
-#endif
+#endif /* __STL_FUNCTION_TMPL_PARTIAL_ORDER */
+#endif /* __STL_NON_TYPE_TMPL_PARAM_BUG */
 
 #if defined(__sgi) && !defined(__GNUC__) && (_MIPS_SIM != _MIPS_SIM_ABI32)
 #pragma reset woff 1174
@@ -1725,7 +1741,7 @@ swap(deque<_Tp,_Alloc,__bufsiz>& __x, deque<_Tp,_Alloc,__bufsiz>& __y)
           
 __STL_END_NAMESPACE 
   
-#endif /* _CPP_BITS_STL_DEQUE_H */
+#endif /* __SGI_STL_INTERNAL_DEQUE_H */
 
 // Local Variables:
 // mode:C++
