@@ -11,41 +11,36 @@
  * purpose.  It is provided "as is" without express or implied warranty.
  */ 
 
-#ifndef _CPP_BITS_STL_STRING_FWD_H
-#define _CPP_BITS_STL_STRING_FWD_H 1
+#ifndef __SGI_STL_STRING_FWD_H
+#define __SGI_STL_STRING_FWD_H
 
 #include <bits/std_cstddef.h>
-
+#include <bits/stl_alloc.h>
 
 __STL_BEGIN_NAMESPACE
 
-#ifdef __STL_USE_STD_ALLOCATORS
-
-template <class _Tp> class allocator;
-
-#else /* __STL_USE_STD_ALLOCATORS */
-
-template <bool __threads, int __inst> class _Default_alloc_template;
-typedef _Default_alloc_template<true, 0> _Alloc;
-
-#endif /* __STL_USE_STD_ALLOCATORS */
-
 template <class _CharT> struct char_traits;
 template <class _CharT, 
           class _Traits = char_traits<_CharT>, 
           class _Alloc = __STL_DEFAULT_ALLOCATOR(_CharT) >
 class basic_string;
 
-typedef basic_string<char> string;
+typedef basic_string<char>    string;
 typedef basic_string<wchar_t> wstring;
 
-template <class _CharT, class _Traits, class _Alloc>
-void _S_string_copy(const basic_string<_CharT,_Traits,_Alloc>&, 
-                    _CharT*, typename _Alloc::size_type);
+const char* __get_c_string(const string&);
+
 __STL_END_NAMESPACE
 
-#endif /* _CPP_BITS_STL_STRING_FWD_H */
+#endif /* __SGI_STL_STRING_FWD_H */
 
 // Local Variables:
 // mode:C++
 // End:
+
+
+
+
+
+
+
